import React from "react";
import {createStore} from "redux";
import rootReducer from "../reducers";
import {Provider} from "react-redux";
import App from "../App";
import Enzyme, {mount} from "enzyme";
import Adapter from "enzyme-adapter-react-16";
import * as Actions from "../actions";

Enzyme.configure({
    adapter: new Adapter()
});

export const testStore = () => {
    return createStore(
        rootReducer,
    )
};

const setup = (store) => {
    return mount(
        <Provider store={store}>
            <App/>
        </Provider>
    )
};

describe("test_action_creators", () => {
    let store;
    let App;
    beforeEach(() => {
        store = testStore();
        App = setup(store);
        store.dispatch(Actions.setGridWidth(15));
        store.dispatch(Actions.setGridHeight(15));
    });

    test("sample_test_1", () => {
        jest.useFakeTimers();

        store.dispatch(Actions.toggleAlive(5, 5));
        store.dispatch(Actions.toggleAlive(4, 4));
        store.dispatch(Actions.toggleAlive(3, 3));
        store.dispatch(Actions.toggleAlive(2, 2));
        store.dispatch(Actions.toggleAlive(1, 1));
        store.dispatch(Actions.toggleAlive(1, 1));
        store.dispatch(Actions.toggleAlive(0, 0));

        App.find("#toggle-play").at(0).simulate("click");
        expect(store.getState().status.isRunning).toBe(true);
        jest.advanceTimersByTime(50 * 1);
        App.find("#toggle-play").at(0).simulate("click");
        expect(store.getState().grid).toMatchSnapshot();
        expect(store.getState().status.isRunning).toBe(false);
    });

    test("sample_test_2", ()=>{
        store.dispatch(Actions.toggleAlive(2, 6));
        store.dispatch(Actions.toggleAlive(1, 6));
        store.dispatch(Actions.toggleAlive(3, 7));
        store.dispatch(Actions.toggleAlive(4, 8));
        store.dispatch(Actions.toggleAlive(5, 8));
        store.dispatch(Actions.toggleAlive(2, 7));
        store.dispatch(Actions.toggleAlive(1, 7));

        for (let i = 0; i < 10; i++) {
            App.find("#tick").at(0).simulate("click");
        }
        expect(store.getState()).toMatchSnapshot();
    })
});